use crate::components::achievement::AchievementComponent;
use crate::components::milestone::MilestoneComponent;
use crate::event_bus::EventBus;
use crate::services::websocket::WebsocketService;
use std::rc::Rc;
use yew::prelude::*;
use yew_agent::Bridge;
use yew_agent::Bridged;
use yew_router::scope_ext::RouterScopeExt;

pub struct Root {
    _wss: WebsocketService,
    _producer: Box<dyn Bridge<EventBus>>,

    /// None until the first websocket message is received.
    state: Option<common::State>,
}

pub enum Msg {
    HandleMsg(String),
}

impl Component for Root {
    type Message = Msg;
    type Properties = ();

    fn create(ctx: &Context<Self>) -> Self {
        let wss = WebsocketService::new();

        let cb = {
            let link = ctx.link().clone();
            move |e| link.send_message(Msg::HandleMsg(e))
        };

        Self {
            _wss: wss,
            _producer: EventBus::bridge(Rc::new(cb)),
            state: None,
        }
    }

    fn update(&mut self, _ctx: &Context<Self>, msg: Self::Message) -> bool {
        match msg {
            Msg::HandleMsg(s) => {
                let msg: common::WebSocketMessage = serde_json::from_str(&s).unwrap();
                self.state = Some(msg);
                true
            }
        }
    }

    fn view(&self, ctx: &Context<Self>) -> Html {
        let nav = ctx.link().navigator().unwrap();

        let onclick_create_achievement = {
            let nav = nav.clone();
            Callback::from(move |_: web_sys::MouseEvent| {
                nav.push(&crate::Route::CreateAchievement);
            })
        };

        let onclick_create_milestone = Callback::from(move |_: web_sys::MouseEvent| {
            nav.push(&crate::Route::CreateMilestone);
        });

        let Some(state) = &self.state else {
            return html! {
                <p>{"loading..."}</p>
            };
        };

        let achievements = state
            .achievements
            .iter()
            .cloned()
            .map(|a| {
                html! {
                    <AchievementComponent achievement={a} />
                }
            })
            .collect::<Html>();

        let completed_achievements = state.achievements.iter().filter(|a| a.completed).count();

        let mut milestones = state.milestones.clone();
        milestones.sort_by_key(|m| m.goal);
        let milestones = milestones.into_iter().map(|m| html! { <MilestoneComponent milestone={m} completed_achievements={completed_achievements} /> }).collect::<Html>();

        html! {
          <>

          <h1>{"Polter A"}<span style="color: var(--danger-color); text-decoration: line-through">{"bend"}</span>{"chievements!"}</h1>

          <hr />
          <div class="row flex flex-wrap">
            <div class="flex-grow">
              <h3>{"Milestones"}</h3>
            </div>

            <div class="flex-intrinsic-size">
              <button onclick={onclick_create_milestone} class="button-primary">{"New"}</button>
            </div>
          </div>
          {milestones}

          <hr />
          <div class="row flex flex-wrap">
            <div class="flex-grow">
              <h3>{"Achievements"}</h3>
            </div>

            <div class="flex-intrinsic-size">
              <button onclick={onclick_create_achievement} class="button-primary">{"New"}</button>
            </div>
          </div>
          {achievements}

          </>
        }
    }
}
